/*
    Fichier deplacemail.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme dplace vers une autre boite aux lettres les mails
    en attente sur le serveur de messagerie.

    Une option permet de conserver les mails slectionns dans la
    boite aux lettres initiale. Dans ce cas, les mails slectionns
    sont seulement copis.

    On peut choisir de dplacer :
    - tous les mails (fonctionnement par dfaut)
    - un mail particulier identifi par son numro

    Un fichier de configuration est utilis pour se connecter 
    la boite aux lettres, pour connaitre l'adresse d'expdition
    et pour slectionner le serveur SMTP servant  la rexpdition
    des mails.
    Il s'agit d'une variante des fichiers de configuration utiliss
    par la plupart des autres outils de libremail.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "smtp.h"
#include "trtentete.h"
#include "copiedepl.h"


/* programme principal */

int main (int nbarg, char *varg[])
{
    char serv_smtp [80]; // nom du serveur smtp utilis
    FILE *fconf;         // descripteur du fichier de configuration
    int  numail, nbmes;  // numro du mail courant et nombre de mails
    int  optc;           // option : dtermine si on conserve les mails dplacs

    // message signalant la copie ou le dplacement d'un mail
    char mess_copdepl [50];


    // rcupration du nom de l'excutable
    memcom (*varg);

    // se positionner sur le premier argument de deplacemail
    varg ++;

    // rcupration de l'option de fonctionnement ventuelle
    if (--nbarg > 1 && **varg == '-')
    {
        // si option -c
        if (varg [0][1] == 'c')
            // les mails seront galement conservs sur le serveur initial
            optc = 1;
        else
        {
            // "Option %s incorrecte"
            aff_err_arg ("ERR_OPTION", *varg);
            nbarg = 0;   // pour afficher la syntaxe de la commande
        }

        // dcompter l'argument trait
        varg ++;
        nbarg --;
    }
    else
        // pas d'option -c : mails dplacs supprims du serveur initial
        optc = 0;

    // rcuprer le numro du mail  traiter s'il a t spcifi
    if (nbarg && atoi (*varg) > 0)
    {
        numail = atoi (*varg);

        // dcompter l'argument trait
        nbarg --;
        varg ++;
    }
    else
        numail = 0;  // on traitera tous les mails

    // controle du nombre d'arguments restant
    if (nbarg == 1)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (*varg);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcuprer et mmoriser les adresses
                // Email d'expdition et de destination
                mem_adr (adr_exped, fconf);
                mem_adr (adr_dest, fconf);

                // rcupration du nombre de mails
                nbmes = nbmails ();

                // si des mails peuvent tre traits
                if (nbmes)
                {
                    // recuprer le nom du serveur smtp
                    fgets (serv_smtp, 80, fconf);
                    serv_smtp [strlen (serv_smtp) - 1] = '\0';

                    // connexion au serveur smtp pour l'envoi des mails
                    if (connect_smtp (serv_smtp))
                    {
                        // mmorisation message adapt au traitement des mails
                        if (optc)
                            // "Copie du mail numro %d"
                            strcpy (mess_copdepl, message ("COPIE_MAIL"));
                        else
                            // "Dplacement du mail numro %d\n"
                            strcpy (mess_copdepl, message ("DEPL_MAIL"));

                        // si demande copie ou dplacement d'un mail particulier
                        if (numail)
                        {
                            // si un mail existe avec ce numro
                            if (numail <= nbmes)
                            {
                                // message indiquant copie ou dplacement mail
                                printf (mess_copdepl, numail);

                                // envoyer une copie du mail  l'autre adresse
                                copiemail (numail);

                                // si dplacement des mails
                                if (! optc)
                                    // supprimer le mail du premier serveur
                                    detruitmail (numail);
                            }
                            // sinon, afficher un message d'erreur
                            else
                                // "Numro de mail trop grand"
                                affiche_err ("NUMAIL_TROP_GRAND");
                        }
                        // sinon, on traitera tous les mails
                        else
                        {
                            // pour chaque mail du serveur
                            for (numail = 1; numail <= nbmes; numail++)
                            {
                                // message indiquant copie ou dplacement mail
                                printf (mess_copdepl, numail);
                                fflush (stdout);

                                // envoyer une copie du mail  l'autre adresse
                                copiemail (numail);

                                // si dplacement des mails
                                if (! optc)
                                    // supprimer le mail du premier serveur
                                    detruitmail (numail);
                            }
                        }

                        // pour l'affichage
                        putchar ('\n');

                        // fermeture de la connexion smtp
                        deconnect_smtp ();
                    }
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
        // "Syntaxe : %s [-c] [numro_mail] fichier_configuration"
        psyntaxe ("SYNT_DEPLACEMAIL");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}
