/*
    Fichier entetemail.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme affiche les caractristique de chaque
    fichier mail d'un rpertoire.

    Par dfaut, on analyse les mails du rpertoire courant.
    On peut aussi indiquer un autre rpertoire de recherche.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include "messages.h"
#include "buflect.h"
#include "fmail.h"
#include "trtentete.h"


/* prototypes */
void aff_entete (char *nomfic);


/* variable globale au source
   (pour viter un passage de paramtre) */

int optm = 0;  // option pour traiter les champs d'entte sur plusieurs lignes


/* programme principal */

int main (int nbarg, char *varg[])
{
    DIR *repert;
    struct dirent *entree;


    // rcupration du nom de l'excutable
    memcom (*varg);

    // rcupration ventuelle de l'option -m
    if (--nbarg > 0)
    {
        // si option -m
        if (strcmp (varg [1], "-m") == 0)
        {
            // la mmoriser
            optm = 1;

            // et passer au paramtre suivant
            varg ++;
            nbarg --;
        }
    }

    // rcupration ventuelle du nom du rpertoire des mails
    // ce nom ne doit pas tre constitu d'un - suivi d'un seul autre caractre
    if (nbarg == 1 && (varg [1][0] != '-' || varg [1][2] != '\0'))
    {
        // positionnement dans ce rpertoire
        if (chdir (varg [1]) < 0)
        {
            // "Rpertoire %s inexistant"
            aff_err_arg ("REPERT_INEXISTANT", varg [1]);
            return (0);
        }

        // un paramtre restant de moins
        nbarg --;
    }

    // si on a trait tous les paramtres
    if (nbarg == 0)
    {
        // accder au rpertoire
        repert = opendir (".");

        // lire un nom de fichier
        entree = readdir (repert);

        // tant qu'il y a des noms de fichier  lire
        while (entree)
        {
            // afficher l'entte du fichier aprs un controle son nom
            if (strlen (entree->d_name) == 8 ||
               (strlen (entree->d_name) == 10 && entree->d_name [8] == '.'))
                aff_entete (entree->d_name);

            // et lire le nom du fichier suivant
            entree = readdir (repert);
        }
    }
    else
        // "Syntaxe : %s [-m] [rpertoire]"
        psyntaxe ("SYNT_ENTETEMAIL");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}


/* lit l'entte d'un message et affiche certaines lignes */

void aff_entete (char *nomfic)
{
    int i;  // simple compteur


    // ouverture du fichier mail
    fmail = fopen (nomfic, "r");

    // arrt si erreur imprvue
    if (fmail == 0)
    {
        // "Impossible d'ouvrir le fichier %s"
        aff_err_arg ("IMPOS_OUVR_FICH", nomfic);
        return;
    }

    // lire la premire ligne de l'entte du message
    lire_fmail ();

    // lecture et mmorisation des caractristiques du message
    do
    {
        // affichage de la ligne prcde du nom de fichier
        majlignentete ();
        printf ("%s:%s", nomfic, buf_lect);

        // lire la ligne suivante
        lire_fmail ();

        // tant que cette ligne est la suite du champ courant
        while (*buf_lect == ' ' || *buf_lect == '\t')
        {
            // si option -m
            if (optm)
            {
                // affichage de la ligne
                majlignentete ();
                printf ("%s", buf_lect);
            }

            // lire la ligne suivante
            lire_fmail ();
        }

        // fin d'affichage de la ligne d'entte
         putchar ('\n');
    }
    while (buf_lect [0] != '\0');  // lecture entte termine si ligne vide

    // librer le fichier mail
    fclose (fmail);
}
