/*
    Fichier messages.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Bibliothque de fonctions utilisant des fichiers de donnes
    pour dialoguer avec l'utilisateur dans sa langue.
*/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "messages.h"
#include "szchemin.h"


// #define DEBUG


#define  szlig   120  // longueur maximale des lignes des fichiers de donnes



/* variables globales au source */

char *ncom;  // nom de (ou chemin d'accs ) la commande en cours d'excution
char repert_com [szchemin] = ""; // rpertoire o est implante cette commande



/* mmorise le nom ou le chemin d'accs de la commande en cours d'excution */

void memcom (char *arg0)
{
    ncom = arg0;

    // si le chemin d'accs  la commande est un chemin relatif
    if (*ncom != '/')
        // chercher le chemin d'accs absolu avant un ventuel
        // chdir dans le programme, on vitera ainsi une erreur
        // d'ouverture ultrieure du fichier des messages
        dircom ();
}



/* retourne le nom ou le chemin d'accs de la commande en cours d'excution */

char *nomcom ()
{
    return (ncom);
}



/* retourne le nom du rpertoire o est la commande en cours d'excution */

char *dircom ()
{
    int derslash;  // position du dernier slash rencontr dans ncom
    char *path;    // variable d'environnement PATH
    char chemcom [szchemin]; // chemin d'accs complet  la commande
    int  i, j;     // indices pour parcourir et recopier ncom et path


    // si le nom de ce rpertoire n'est pas encore mmoris
    if (! *repert_com)
    {
        // initialisation
        derslash = 0;
        i = 0;
        j = 0;

        // recopier ncom dans repert_com et chercher la position du dernier /
        while (ncom [i])
        {
            // on saute les ./ trouvs pendant la recopie
            while (ncom [i] == '.' && ncom [i+1] == '/')
                i = i + 2;

            // recopier un nom de rpertoire (ou le nom du fichier de commande)
            while (ncom [i] != '/' && ncom [i])
                repert_com [j++] = ncom [i++];

            // si on arrive sur un /
            if (ncom [i] == '/')
            {
                // mmoriser la position du / dans repert_com
                derslash = j;

                // et le recopier
                repert_com [j++] = ncom [i++];
            }
        }

        // on ne conserve dans repert_com que le nom
        // du rpertoire o est implante la commande
        repert_com [derslash] = '\0';

        // si aucun nom de rpertoire n'a t trouv
        if (! *repert_com)
        {
            // on va tester les chemins de la variable d'environnement PATH
            path = getenv ("PATH");

            i = 0;

            do
            {
                j = 0;

                // recopier l'un de ces chemins
                while (path [i] != ':' && path [i])
                {
                    repert_com [j++] = path [i++];
                }

                // terminer la chaine de caractres
                repert_com [j] = '\0';

                // gnrer le chemin d'accs  la commande
                sprintf (chemcom, "%s/%s", repert_com, ncom);
            }
            // termin si ce chemin d'accs est le bon
            while (access (chemcom, X_OK) != 0 && path [i++]);
        }

        // si le chemin d'accs trouv est un chemin d'accs relatif
        // on va le transformer en chemin d'accs absolu.
        // la transformation optimise est complexe, mais ncessaire
        // pour le cas ou le programme fait un chdir avant d'ouvrir
        // les fichier des messages ou un autre fichier de donnes
        if (*repert_com != '/')
        {
            // rcuprer le nom du rpertoire courant
            strcpy (chemcom, repert_com);
            getcwd (repert_com, szchemin);

            // on va y accoler le chemin d'accs relatif au
            // rpertoire trouv en traitant les ../

            // initialisation
            i = 0;
            j = strlen (repert_com);

            // on rajoute / au rpertoire courant
            repert_com [j++] = '/';
            repert_com [j]  = '\0';

            while (chemcom [i])
            {
                // si on a le rpertoire ..
                if (chemcom [i] == '.' && chemcom [i+1] == '.'
                       && (chemcom [i+2] == '/' || chemcom [i+2] == '\0'))
                {
                    // se positionner aprs le .. terminal
                    i = i + 2;

                    // ou aprs le ../
                    if (chemcom [i] == '/')
                        i++;

                    // remonter d'un rpertoire dans repert_com
                    do
                        j--;
                    while (j > 1 && repert_com [j-1] != '/');

                    // on ne supprime pas le premier /
                    if (j == 0)
                        j++;
                }
                // sinon (cas gnral)
                else
                {
                    // recopier un nom de rpertoire
                    while (chemcom [i] != '/' && chemcom [i] && j < szchemin)
                        repert_com [j++] = chemcom [i++];

                    repert_com [j++] = chemcom [i++];
                }
            }
        }

        j = strlen (repert_com);

        // enlever les /. terminaux
        while (j > 1 && repert_com [j-1] == '.' && repert_com [j-2] == '/')
            j = j - 2;

        repert_com [j] = '\0';
    }

    // retourner le nom du rpertoire
    return (repert_com);
}



/* retourne la langue de l'utilisateur dans une chaine de 2 caractres */

char *getlang ()
{
    char *langue;           // contenu de la variable d'environnement $LANG
    static char retour [3]; // chaine de caractres retourne


    // rcuprer la variable d'environnement $LANG
    langue = (char *) getenv ("LANG");

    // si cette variable d'environnement a t dclare
    if (langue)
    {
        // on retournera les 2 premiers caractres de cette variable
        retour [0] = langue [0];
        retour [1] = langue [1];
        retour [2] = '\0';
    }
    // sinon
    else
        // on retournera une chaine vide
        *retour = '\0';

    // retourner le rsultat
    return (retour);
}



/* vrifie si le jeu de caractres UTF-8 est utilis */

int util_utf8 ()
{
    static int testacces; // mmorise si on a dj appel cette fonction
    static int restest;   // rsultat de la vrification
    char       *langue;   // caractre de la variable d'environnement $LANG
    int        lg_langue; // longueur de la chaine $LANG


    // si on appelle cette fonction pour la premire fois
    if (! testacces)
    {
        // rcuprer la variable d'environnement $LANG
        langue = (char *) getenv ("LANG");

        // initialisation
        restest = 0;

        // si cette variable d'environnement est initialise et se
        // termine par UTF-8, ou UTF8 en majuscules ou minuscules
        // alors on utilise ce jeu de caractres
        if (langue) // && strlen (langue) > 5)
        {
            // compter le nombre de caractres de $LANG
            lg_langue = 0;

            while (*langue)
            {
                langue ++;
                lg_langue ++;
            }

            // et se positionner sur le dernier
            langue --;

            // si $LANG assez long et se termine par '8'
            if (lg_langue > 5 && *langue == '8')
            {
                // sauter le '-' prcdent ventuel
                if (*(--langue) == '-')
                    langue --;

                // vrifier et mmoriser si chaine "utf" ou "UTF"
                if ((*(langue-2) | 0x20) == 'u' && (*(langue-1) | 0x20) == 't'
                                                && (*langue | 0x20) == 'f')
                    restest = 1;
            }
        }

        // on ne repassera plus dans ce bloc d'instructions
        testacces = 1;
    }

    // retourner le rsultat du test
    return restest;
}



/* retourne le chemin d'accs  un fichier de donnes
   aprs avoir choisi le fichier le mieux adapt compte tenu :
   - de la langue de l'utilisateur
   - de l'indicateur de prise en compte du jeu de caractres utf-8
   - de la prsence ou non du fichier deflang-libremail
 */

char *chemfichlang (char *nomfic, int utf8)
{
    static char chemfic [szchemin]; // chemin d'accs au fichier de donnes
    char ficdeflang [szchemin];     // chemin d'accs au fichier "deflang-..."
    char *pos_suffixe;              // posit 1er caract du suffixe dans chemfic
    char langue [3];                // langue de l'utilisateur
    int  sans_suff, avec_suff;      // indicateur de prsence des fichiers


    // gnrer le chemin d'accs au fichier sans suffixe

    // si libremail est implant localement
    if (strcmp (dircom (), "/usr/bin") != 0)
        // fichiers de donnes dans le mme rpertoire que les commandes
        sprintf (chemfic, "%s/%s", dircom (), nomfic);
    // sinon, les commandes de libremail sont dans /usr/bin
    else
        // et les fichiers de donnes dans /usr/share/libremail
        sprintf (chemfic, "/usr/share/libremail/%s", nomfic);

    // rcuprer la langue de l'utilisateur
    strcpy (langue, getlang ());

    // si une langue a t dfinie
    if (*langue)
    {
        // mmoriser la possibilit d'accs en lecture du fichier sans suffixe
        sans_suff = (access (chemfic, R_OK) == 0);

        // chercher la position o rajouter un suffixe au fichier
        pos_suffixe = chemfic + strlen (chemfic);

        // si jeu de caractres UTF-8 utilis et  prendre en compte
        if (utf8)
        {
            // rajouter au chemin d'accs la langue suivie de "-utf"
            sprintf (pos_suffixe, ".%s-utf", langue);

            // si ce fichier n'existe pas ou n'est pas accessible en lecture
            if (access (chemfic, R_OK) < 0)
                // supprimer le "-utf"  la fin du chemin d'accs
                pos_suffixe [3] = '\0';
        }
        // sinon
        else
            // rajouter au chemin d'accs l'indication de la langue
            sprintf (pos_suffixe, ".%s", langue);

        // mmoriser la possibilit d'accs en lecture du fichier avec suffixe
        avec_suff = (access (chemfic, R_OK) == 0);

        // si les fichiers avec et sans suffixe sont tous
        // deux accessibles en lecture, ou si aucun ne l'est
        if (sans_suff == avec_suff)
        {
            // crer le chemin d'accs au fichier "deflang-libremail"
            sprintf (ficdeflang, "%s/deflang-libremail", dircom ());

            // si ce fichier existe
            if (access (ficdeflang, F_OK) == 0)
                // revenir au nom de fichier sans suffixe
                *pos_suffixe = '\0';

            // sinon, on utilisera le nom de fichier avec suffixe
        }
        // sinon si le fichier sans suffixe est le seul accessible en lecture
        else if (sans_suff && (! avec_suff))
            // revenir au nom de fichier sans suffixe
            *pos_suffixe = '\0';

        // sinon le fichier avec suffixe est le seul accessible en lecture
        // son nom est dj mmoris dans chemfic
    }

    // retourner le chemin d'accs au fichier
    return chemfic;
}



/* retourne le nom du fichier ou du rpertoire associ au mnmonique */

char *ficdir (char *mnemonique)
{
    static int testacces = 0;    // mmorise si on a dj appel cette fonction
    char chem_ficdir [szchemin]; // chemin d'accs au fichier ficdir-libremail
    static FILE *desc_ficdir;    // descripteur du fichier ficdir-libremail
    char ligne [szlig];          // contenu d'une ligne du fichier
    static char nom_ficdir [szmax_ficdir]; // nom associ au mnmonique
    int  i, j;                   // indices pour traitements chaines caractres


#ifdef DEBUG
    printf ("Appel de ficdir (%s)\n", mnemonique);
    sleep (1);
#endif

    // si on appelle cette fonction pour la premire fois
    if (! testacces)
    {
        // gnrer le chemin d'accs au fichier ficdir-libremail
        // en tenant compte de la langue
        strcpy (chem_ficdir, chemfichlang ("ficdir-libremail", 0));

        // tenter de l'ouvrir en lecture
        desc_ficdir = fopen (chem_ficdir, "r");

        // on ne repassera plus dans ce bloc d'instructions
        testacces = 1;
    }

    // si le fichier ficdir-libremail a pu tre ouvert
    if (desc_ficdir)
    {
        // se positionner au dbut de ce fichier
        rewind (desc_ficdir);

        // on va l'explorer pour trouver le mnmonique
        while (fgets (ligne, szlig, desc_ficdir))
        {
            // comparaison du mnmonique de la ligne
            // lue avec celui pass en paramtre
            i = 0;

            while (ligne [i] == mnemonique [i])
                i++;

            // si mnmonique trouv
            if (mnemonique [i] == '\0')
            {
                // se positionner sur le nom correspondant
                while (ligne [i] < 'a' && ligne [i])
                    i++;

                j = 0;

                // on va recopier ce nom
                while (ligne [i] > '#')
                {
                    nom_ficdir [j++] = ligne [i++];

                    // en vitant un dbordement de tableau
                    if (j >= szmax_ficdir)
                // "Lors de l'appel de ficdir (%s), le nom obtenu est trop long"
                        errfatale ("DEBORDE_FICDIR", mnemonique);
                }

                // terminer le nom
                nom_ficdir [j] = '\0';

                // retourner le nom trouv (s'il n'est pas vide !)
                if (*nom_ficdir)
                    return (nom_ficdir);
            }
        }
    }

    // mnmonique non trouv dans un fichier, on va gnrer le nom par dfaut

    // initialisation compteurs
    i = 0;
    j = 0;

    // sauter le prfixe DIR_ ou FIC_ du mnmonique
    while (mnemonique [i++] != '_' && mnemonique [i])
        ;   // passage au caractre suivant

    // recopier la fin du mnmonique
    do
    {
        // en vitant un dbordement de tableau
        if (j >= szmax_ficdir)
            // "Lors de l'appel de ficdir (%s), le nom obtenu est trop long"
            errfatale ("DEBORDE_FICDIR", mnemonique);

        // en passant les caractres en minuscules
        nom_ficdir [j++] = tolower (mnemonique [i]);
    }
    while (mnemonique [i++]);

    // retourner le nom que l'on vient de gnrer
    return (nom_ficdir);
}



/* sortie du programme sur impossibilit d'afficher un message */

void erreur_mess (int numerr, char *info)
{
    // messages mmoriss en dur dans le logiciel dans des tableaux
    // statiques pour viter allocation mmoire et initialisation

    static char *fichmanquant [] =
    {
         "fr Fichier %s manquant ou protg en lecture",
         "de In Lektre geschtzte oder fehlende Kartei %s",
         "en File %s missing or read protected",
         "eo Dosiero %s mankante au protektita en lego",
         "es Fichero %s que falta o protegido en lectura",
         "it Archivio %s che manca o protetto in lettura",
         "nl Bestand dat %s of dat in lezing wordt beschermd gebrek heeft aan",
         "pt Ficheiro %s que falta ou protegido em leitura",
         NULL   // pour terminer la liste
    };

    static char *mnemomanquant [] =
    {
         "fr Mnmonique MNEMO_ABSENT manquant dans le fichier %s",
         "de MNEMO_ABSENT Mnemonik, die in der Kartei %s fehlt",
         "en Mnemonic MNEMO_ABSENT missing in the file %s",
         "eo Mnemoniko MNEMO_ABSENT mankante en la dosiero %s",
         "es Mnemotecnia MNEMO_ABSENT que falta en el fichero %s",
         "it Mnmonique MNEMO_ABSENT che manca nell'archivio %s",
         "nl Mnmonique MNEMO_ABSENT die in het bestand %s gebrek heeft aan",
         "pt Mnmonique MNEMO_ABSENT que falta no ficheiro %s",
         NULL   // pour terminer la liste
    };

    char **listemessages;   // pour slectionner une liste de messages
    char langue [3];        // langue de l'utilisateur
    char mess_err [szlig];  // contenu du message rcupr
    int  i;                 // compteur dans la liste des messages


    // slectionner la bonne liste de messages
    if (numerr == 1)
        listemessages = fichmanquant;
    else
        listemessages = mnemomanquant;

    // rcuprer la langue de l'utilisateur
    strcpy (langue, getlang ());

    // rechercher le message dans la bonne langue
    i = 0;

    while (listemessages [i] && (listemessages [i][0] != langue [0]
                              || listemessages [i][1] != langue [1]))
        i++;

    // si le message a t trouv
    if (listemessages [i])
    {
        // le mettre en forme
        sprintf (mess_err, listemessages [i] + 3, info);

        // si on utilise l'encodage UTF-8
        if (util_utf8 ())
            // convertir les caractres accentus du message
            conv_iso_utf8 (mess_err);

        // afficher le message d'erreur
        fprintf (stderr, "\n%s\n", mess_err);
    }
    // sinon
    else
    {
        // on va afficher le message dans toutes les langues disponibles
        i = 0;

        while (listemessages [i])
        {
            // rcuprer un message et le mettre en forme
            sprintf (mess_err, listemessages [i++] + 3, info);

            // si on utilise l'encodage UTF-8 convertir les caractres accentus
            if (util_utf8 ())
                conv_iso_utf8 (mess_err);

            // afficher le message d'erreur
            fprintf (stderr, "\n%s\n", mess_err);
        }
    }

    // sortir du programme
    exit (-1);
}



/* retourne le message associ au mnmonique pass en paramtre */

char *message (char *mnemonique)
{
    static int testacces = 0;    // mmorise si on a dj appel cette fonction
    static char ficmes [szchemin]; // chemin d'accs au fichier mess-libremail
    static FILE *desc_ficmes;    // descripteur du fichier mess-libremail
    static char mess_lu [szlig]; // message rcupr dans le fichier
    static char old_mnemo [szlig]; // mnmonique lors de l'appel prcdent
    char ligne [szlig];          // contenu d'une ligne du fichier
    int  retourdeb;              // mmorise si retour en dbut de fichier
    int  i, j;                   // indices pour traitements chaines caractres


#ifdef DEBUG
    printf ("Appel de message (%s)\n", mnemonique);
    sleep (1);
#endif

    // si on appelle cette fonction pour la premire fois
    if (! testacces)
    {
        // gnrer le chemin d'accs au fichier mess-libremail
        // en tenant compte de la langue et du jeu de caractres
        strcpy (ficmes, chemfichlang ("mess-libremail", util_utf8 ()));

        // tenter de l'ouvrir en lecture
        desc_ficmes = fopen (ficmes, "r");

        // si le fichier mess-libremail n'a pas pu tre ouvert
        if (!desc_ficmes)
            // message d'erreur fatale
            // "Fichier %s manquant ou protg en lecture"
            erreur_mess (1, ficmes);

        // on ne repassera plus dans ce bloc d'instructions
        testacces = 1;

        // on est au dbut du fichier
        retourdeb = 1;
    }
    // sinon
    else
        // le fichier sera explor  partir de la dernire chaine trouve
        retourdeb = 0;

    // si on redemande le mme mnmonique que la fois prcdente
    if (strcmp (mnemonique, old_mnemo) == 0)
        // retourner directement le message trouv prcdemment
        return (mess_lu);
    else
        // sinon, mmoriser ce mnmonique
        strcpy (old_mnemo, mnemonique);

    // le fichier ficmes-libremail a pu tre ouvert
    // on va l'explorer pour trouver le mnmonique

    // premier essai  partir de la ligne qui suit celle du message prcdent
    fgets (ligne, szlig, desc_ficmes);

    do
    {
        // comparaison du mnmonique de la ligne
        // lue avec celui pass en paramtre
        i = 0;

        while (ligne [i] == mnemonique [i])
            i++;

        // si mnmonique trouv
        if (mnemonique [i] == '\0' && (ligne [i] == ' ' || ligne [i] == '\t'))
        {
            // se positionner sur le message correspondant
            while (ligne [i] != '"' && ligne [i] != '\n' && ligne [i])
                i++;

            j = 0;
            i++;

            // on va recopier le message
            while (ligne [i] != '"' && ligne [i] != '\n' && ligne [i])
            {
                // si \ dans le message, on convertit le caractre spcial
                if (ligne [i] == '\\')
                {
                    switch (ligne [++i])
                    {
                        case 'n' : mess_lu [j++] = '\n';
                                   break;

                        case 'r' : mess_lu [j++] = '\r';
                                   break;

                        case 't' : mess_lu [j++] = '\t';
                                   break;

                        case '\\': mess_lu [j++] = '\\';
                                   break;

                        default  : mess_lu [j++] = '\\';
                                   mess_lu [j++] = ligne [i];
                    }

                    // et on passe au caractre suivant
                    i++;
                }
                // sinon, simple copie du caractre
                else
                    mess_lu [j++] = ligne [i++];
            }

            // terminer le message
            mess_lu [j] = '\0';

            // si on utilise l'encodage UTF-8
            if (util_utf8 ())
                // convertir les caractres accentus du message
                conv_iso_utf8 (mess_lu);

            // retourner le message trouv
            return (mess_lu);
        }

        // si on vient de tester sans succs la chaine du fichier
        // qui suit celle du message demand juste auparavant
        if (! retourdeb)
        {
            // reprendre la recherche en dbut de fichier
            rewind (desc_ficmes);

            // on explorera le fichier jusqu' la fin si ncessaire
            retourdeb = 1;
        }
    }
    // continuer la recherche ligne par ligne jusqu'en fin de fichier
    while (fgets (ligne, szlig, desc_ficmes));

    // mnmonique non trouv dans le fichier mess-libremail
    if (strcmp ("MNEMO_ABSENT", mnemonique) != 0)
        // sortie sur message d'erreur
        errfatale ("MNEMO_ABSENT", mnemonique);
    else
        // "Mnmonique MNEMO_ABSENT manquant dans le fichier des messages"
        erreur_mess (2, ficmes);
}



/* convertit les caractres spciaux du message
   pass en paramtre dans le format UTF-8 */
void conv_iso_utf8 (char *mess)
{
    int i, j;  // compteurs de caractres


    // initialisation
    i = 0;
    j = 0;

    // compter les caractres spciaux
    while (mess [i])
    {
        if (mess [i++] & 0x80)
            j++;
    }

    // termin si pas de caractres spciaux
    if (j == 0)
        return;

    // dcaler la chaine  convertir de j caractres
    j = j + i;

    while (i >= 0)
        mess [j--] = mess [i--];

    // remettre les caractres  la bonne place
    // en convertissant les caractres spciaux
    do
    {
        // si caractre spcial
        if (mess [++j] & 0x80)
        {
            // on le remplace par les 2 caractres de l'encodage UTF-8
            if (mess [j] & 0x40)
            {
                mess [++i] = 0xC3;
                mess [++i] = mess [j] ^ 0x40;
            }
            else
            {
                mess [++i] = 0xC2;
                mess [++i] = mess [j];
            }
        }
        // sinon
        else
            // recopier le caractre sans conversion
            mess [++i] = mess [j];
    }
    while (mess [j]);
}



/* affichage d'une ligne de texte */

void affiche_msg (char *mnemonique)
{
    puts (message (mnemonique));
}



/* affichage d'une ligne de texte sans retour  la ligne */

void affiche_msg_nocr (char *mnemonique)
{
    printf ("%s", message (mnemonique));
}



/* affichage d'un message d'erreur simple */

void affiche_err (char *mnemonique)
{
    fputs (message (mnemonique), stderr);
    fputc ('\n', stderr);
}



/* affichage d'un message d'erreur avec argument */

void aff_err_arg (char *mnemonique, char *argument)
{
    fprintf (stderr, message (mnemonique), argument);
    fputc ('\n', stderr);
}



/* affichage d'un message d'erreur avec argument numrique */

void aff_err_argnum (char *mnemonique, int argument)
{
    fprintf (stderr, message (mnemonique), argument);
    fputc ('\n', stderr);
}



/* sortie du programme sur erreur fatale */

void errfatale (char *mnemonique, char *argument)
{
    // crire le message d'erreur
    aff_err_arg (mnemonique, argument);

    // sortir du programme
    exit (-1);
}


/* sortie du programme en rappellant la syntaxe d'une commande */

void psyntaxe (char *mnemonique)
{
    errfatale (mnemonique, nomcom ());
}


/* sortie du programme en rappellant la syntaxe d'une commande sur 2 lignes */

void psyntaxe2 (char *mnemonique1, char *mnemonique2)
{
    aff_err_arg (mnemonique1, nomcom ());
    errfatale   (mnemonique2, nomcom ());
}
