/*
    Fichier sjmails.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme affiche les caractristique de chaque mail
    en attente de tlchargement (ou de destruction).

    Il y a 3 formats d'affichage.

    Le format dtaill (option -d) donne plusieurs lignes d'information :
    - date et heure d'envoi
    - expditeur et destinataire(s) + ventuelle adresse de rponse
    - sujet du mail
    - format du message (texte, html ...)
    - ventuellement premires lignes du message

    Le format simplifi (option -s) affiche une ligne par message :
      nom expditeur     sujet     date et heure d'envoi

    La variante du format simplifi (option -e) affiche une ligne par message :
      adresse expditeur     sujet     date et heure d'envoi

    Un fichier de configuration est utilis pour se connecter  la
    boite aux lettres.
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "messages.h"
#include "buflect.h"
#include "ficonf.h"
#include "pop.h"
#include "trtentete.h"
#include "trtligne.h"


// options d'affichage
#define opt_s           0
#define opt_e           1
#define opt_d           2


/* prototypes */
void aff_detail (int numes);
void aff_resume (int numes, int lg_num);


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

int nblig_mes;    // nombre de lignes  afficher pour chaque message
int opt_aff;      // option affichage (simple = -s et -e, dtaill = -d)
int opt_inv = 0;  // option inversion de l'ordre d'affichage des mails
int test_xorig;   // traitement ou non du champ X-Original-From

// largeur des champs affichs en mode simplifi (options -s et -e)
int sz_nom, sz_adr, sz_avantdate;


/* programme principal */

int main (int nbarg, char *varg[])
{
    FILE *fconf;           // descripteur du fichier de configuration
    int  numes, nbmes;     // numro du mail courant et nombre de mails
    int  lg_num;           // nombre caractres pour afficher tous numros mails
    int  sup_nbmes;        // variable pour calculer lg_num
    char format_numes [6]; // format affichage numros mails
    int  largeur;          // largeur de l'affichage si option -s ou -e
    char *varenv_xorig;    // variable d'environnement libremail_xorig


    // rcupration du nom de l'excutable
    memcom (*varg);

    // se positionner sur le premier argument de sjmails
    varg ++;
    nbarg --;

    // initialisation options de fonctionnement
    opt_aff = opt_s;
    nblig_mes = 0;
    largeur = 0;   // la valeur sera fixe plus tard

    // rcupration des options de fonctionnement ventuelles
    while (nbarg > 0 && **varg == '-' && varg [0][2] == '\0')
    {
        switch (varg [0][1])
        {
            // option affichage dtaill
            case 'd' : opt_aff = opt_d;

                       // recupration du nombre de lignes s'il existe
                       nblig_mes = atoi (varg [1]);

                       // dans ce cas, dcompter l'argument trait
                       if (nblig_mes || *varg [1] == '0')
                       {
                           varg ++;
                           nbarg --;
                       }

                       break;

            // option affichage simplifi
            case 's' : opt_aff = opt_s;
                       break;

            // option affichage simplifi avec adresse expditeur
            case 'e' : opt_aff = opt_e;
                       break;

            // option largeur de l'affichage simplifi
            case 'w' : // passer  l'argument suivant
                       varg ++;
                       nbarg --;

                       // rcupration de la largeur de l'affichage
                       largeur = atoi (*varg);
                       break;

            // option inversion de l'ordre d'affichage
            case 'i' :
            case 'r' : opt_inv = 1;
                       break;

            default  : // "Option %s incorrecte"
                       aff_err_arg ("ERR_OPTION", *varg);

                       // rappeller la syntaxe de la commande
                       psyntaxe ("SYNT_SJFMAILS");
        }

        // dcompter l'argument trait
        varg ++;
        nbarg --;
    }

    // controle du nombre d'arguments restants
    if (nbarg == 1)
    {
        // ouvrir le fichier de configuration
        fconf = ouvre_ficonf (varg [0]);

        if (fconf)
        {
            // connexion sur le compte mail du serveur pop
            if (connect_pop (fconf))
            {
                // rcupration du nombre de mails
                nbmes = nbmails ();

                // dterminer si on va traiter les champs X-Original-From
                varenv_xorig = getenv ("libremail_xorig");

                if (varenv_xorig)
                    test_xorig = (tolower (*varenv_xorig) != 'n');
                else
                    test_xorig = 1;

                // affichage dtaill ou rsum de l'entte des mails
                if (opt_aff == opt_d)
                {
                    // l'ordre d'affichage dpend de l'option -i ventuelle
                    if (opt_inv)
                    {
                        // option -i : on affiche les mails
                        // du plus rcent au plus ancien
                        for (numes = nbmes; numes >= 1; numes--)
                            aff_detail (numes);
                    }
                    else
                    {
                        // pas d'option : on affiche les mails
                        // dans leur ordre d'arrive sur le serveur
                        for (numes = 1; numes <= nbmes; numes++)
                            aff_detail (numes);
                    }
                }
                else
                {
                    // auparavant calcul nbre min de car pour n messages
                    lg_num = 1;
                    sup_nbmes = 10;

                    while (nbmes >= sup_nbmes)
                    {
                        lg_num ++;
                        sup_nbmes *= 10;
                    }

                    // et fabrication de la chaine de format d'affichage
                    sprintf (format_numes, "%%%dd ", lg_num++);

                    // valeur par dfaut de la largeur
                    // d'affichage si elle n'a pas t fixe
                    if (! largeur)
                        largeur = 80;

                    // calcul de la taille des diffrentes zones d'affichage
                    sz_nom = (largeur / 4) + lg_num;
                    sz_adr = (largeur / 3) + lg_num - 2;
                    sz_avantdate = largeur - 13;

                    // affichage rsum des enttes de mails
                    // l'ordre d'affichage dpend de l'option -i ventuelle
                    if (opt_inv)
                    {
                        // option -i : on affiche les mails
                        // du plus rcent au plus ancien
                        for (numes = nbmes; numes >= 1; numes--)
                        {
                            printf (format_numes, numes);
                            aff_resume (numes, lg_num);
                        }
                    }
                    else
                    {
                        // pas d'option : on affiche les mails
                        // dans leur ordre d'arrive sur le serveur
                        for (numes = 1; numes <= nbmes; numes++)
                        {
                            printf (format_numes, numes);
                            aff_resume (numes, lg_num);
                        }
                    }
                }

                // se dconnecter proprement du serveur pop
                deconnect_pop ();
            }

            // on n'a plus besoin du fichier de configuration
            fclose (fconf);
        }
    }
    else
        // "Syntaxe : %s [-(s|e|d [nb_lignes_messages])]
        //               [-w largeur] [-i] fichier_configuration"
        psyntaxe ("SYNT_SJMAILS");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* lit l'entte d'un message et affiche certaines lignes */

void aff_detail (int numes)
{
    char bufw [80]; // buffer d'envoi d'une requte
    // buffers pour affichage ordonn des caractristiques du message
    char bufFrom [80], bufTo [80], bufCc [80], bufReply [80],
         bufSubject [80], bufContent [80], bufXorig [80];
    int i, j;       // simples compteurs


    // initialisation
    bufFrom [0]    = '\0';
    bufTo [0]      = '\0';
    bufCc [0]      = '\0';
    bufReply [0]   = '\0';
    bufSubject [0] = '\0';
    bufContent [0] = '\0';
    bufXorig [0]   = '\0';

    // demande de lecture de l'entte du message
    if (nblig_mes)
        sprintf (bufw, "TOP %d %d", numes, nblig_mes);
    else
        sprintf (bufw, "TOP %d 1", numes);

    env_pop (bufw);

    // "Message %d\n"
    putchar ('\n');
    printf (message ("AFF_NUMAIL"), numes);

    // lire la premire ligne de l'entte du message
    lire_pop ();

    // termin pour ce mail si erreur d'envoi cot serveur
    if (memcmp (buf_lect, "-ERR ", 5) == 0)
    {
        puts (buf_lect);
        return;
    }

    // lecture et mmorisation des caractristiques du message
    do
    {
        // si c'est le champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_pop ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
            puts (buf_lect);   // la date est affiche directement
        else if (start ("From"))
            memconvbuf (bufFrom);
        else if (start ("X-Original-From") && test_xorig)
            memconvbuf (bufXorig);
        else if (start ("To"))
            memconvbuf (bufTo);
        else if (start ("Cc"))
            memconvbuf (bufCc);
        else if (start ("Reply-To"))
            memconvbuf (bufReply);
        else if (start ("Content-Type"))
        {
            membuf (bufContent);

            if (recup_ctype () == TextPlain)
                lire_charset ();
        }
        else if (start ("Content-Transfer-Encoding"))
            mem_encodage ();

        // lire la ligne suivante de l'entte du message
        lire_pop ();
    }
    while (buf_lect [0] != '\0');  // lecture entte termine si ligne vide

    // affichage des caractristiques mmorises
    if (bufFrom [0])
        puts (bufFrom);
    if (bufXorig [0])
        puts (bufXorig);
    if (bufTo [0])
        puts (bufTo);
    if (bufCc [0])
        puts (bufCc);
    if (bufReply [0])
        puts (bufReply);
    if (bufSubject [0])
        puts (bufSubject);
    if (bufContent [0])
        puts (bufContent);

    // sauter la ligne blanche qui prcde le message
    lire_pop ();

    // lecture de la fin du message jusqu'a une ligne limite a un .
    while (buf_lect [0] != '.' || buf_lect [1] != '\0')
    {
        // affichage ventuel des dernires lignes lues
        if (nblig_mes)
        {
            // on prend en compte les nouveaux champs Content ventuels
            if (start ("Content-Type") && recup_ctype () == TextPlain)
                lire_charset ();
            else if (start ("Content-Transfer-Encoding"))
                mem_encodage ();

            // convertir la ligne lue et l'afficher
            majligne ();
            printf ("%s", buf_lect);
        }

        // et lecture de la suivante
        lire_pop ();
    }
}



/* lit l'entte d'un message et affiche un rsum sur une ligne */

void aff_resume (int numes, int lg_num)
{
    char bufw [80]; // buffer d'envoi d'une requte
    // buffers pour affichage ordonn des caractristiques du message
    char bufFrom [80], bufSubject [80], bufDate [80];
    // 1re et ventuellement 3me lettre des noms de mois
    static char *mois [] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
                             "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
    int sz_champ1;  // nombre max de caractres du champ expditeur
    int i, j;       // simples compteurs


    // initialisation
    bufFrom [0]    = '\0';
    bufSubject [0] = '\0';
    bufDate [0]    = '\0';

    // demande de lecture de l'entte du message
    if (nblig_mes)
        sprintf (bufw, "TOP %d %d", numes, nblig_mes);
    else
        sprintf (bufw, "TOP %d 1", numes);

    env_pop (bufw);

    // lire la premire ligne de l'entte du message
    lire_pop ();

    // termin pour ce mail si erreur d'envoi cot serveur
    if (memcmp (buf_lect, "-ERR ", 5) == 0)
    {
        puts (buf_lect);
        return;
    }

    // lecture et mmorisation des caractristiques du message
    do
    {
        // si c'est le champ sujet
        if (start ("Subject"))
        {
            // mmoriser la ligne
            memconvbuf (bufSubject);

            // on va tester la ligne qui suit
            lire_pop ();
            majlignentete ();

            // si c'est encore une ligne du sujet
            if (*buf_lect == ' ' || *buf_lect == '\t')
            {
                // initialisation compteurs
                i = 1;
                j = strlen (bufSubject);

                // complter le sujet du mail
                while (buf_lect [i] && j < sizeof (bufSubject) - 1)
                    bufSubject [j++] = buf_lect [i++];

                // terminer la chaine de caractres
                bufSubject [j] = '\0';
            }
        }

        // mmorisation des autres caractristiques importantes
        if (start ("Date"))
            membuf (bufDate);
        else if (start ("From") && ! *bufFrom)
            memconvbuf (bufFrom);
        else if (start ("X-Original-From") && test_xorig)
            memconvbuf (bufFrom);

        // lire la ligne suivante de l'entte du message
        lire_pop ();
    }
    while (buf_lect [0] != '.' || buf_lect [1] != '\0');

    // initialisation : on est en dbut de ligne
    i = lg_num;

    // affichage du nom ou de l'adresse de l'expditeur du message
    if (*bufFrom)
    {
        if (tolower (*bufFrom) == 'x')
            j = 16;
        else
            j = 6;

        // option -s : affichage du nom de l'expditeur
        if (opt_aff == opt_s)
        {
            while (bufFrom [j] == '"' || bufFrom [j] == ' ')
                j++;

            putchar (bufFrom [j++]);

            // si affichage avec un jeu de caractres ISO-8859
            if (! util_utf8 ())
            {
                // on affichera sz_nom caractres maximum
                while (++i < sz_nom && bufFrom [j] && bufFrom [j] != '<'
                             && bufFrom [j] != '"' && bufFrom [j] != '=')
                    putchar (bufFrom [j++]);
            }
            // sinon (affichage avec le jeu de caractres UTF-8)
            else
            {
                // on affichera toujours sz_nom caractres maximum
                while (++i < sz_nom && bufFrom [j] && bufFrom [j] != '<'
                             && bufFrom [j] != '"' && bufFrom [j] != '=')
                {
                    // mais certains caractres ne seront pas compts
                    if ((bufFrom [j] & 0xC0) == 0x80)
                        i--;

                    putchar (bufFrom [j++]);
                }
            }
        }
        // option -e : affichage de l'adresse de l'expditeur
        else
        {
            // recherche de la partie adresse de l'expditeur
            while (bufFrom [j] != '@' && bufFrom [j])
                j++;

            // positionnement au dbut de cette adresse
            while (bufFrom [j] != '<' && bufFrom [j] != ' ')
                j--;

            j++;

            // copie de l'adresse
            putchar (tolower (bufFrom [j++]));

            // pour ne pas tronquer l'adresse, on
            // accepte de dborder sur le champ sujet
            while (++i < sz_avantdate && bufFrom [j]
                                      && bufFrom [j] != '>')
                putchar (tolower (bufFrom [j++]));
        }
    }

    // dtermination nombre de caractres rserv au champ expditeur
    if (opt_aff == opt_s)
        sz_champ1 = sz_nom;
    else
        sz_champ1 = sz_adr;

    // complter ce champ par des blancs
    while (i++ <= sz_champ1)
        putchar (' ');

    // affichage du sujet du message
    if (*bufSubject)
    {
        j = 9;

        // on saute les blancs inutiles
        while (bufSubject [j] == ' ')
            j++;

        // si affichage avec un jeu de caractres ISO-8859
        if (! util_utf8 ())
        {
            // on affichera jusqu' sz_avantdate caractres maximum
            while (++i < sz_avantdate && bufSubject [j])
            {
                // si prsence d'une tabulation
                if (bufSubject [j] == '\t')
                {
                    // afficher un blanc pour une meilleure mise en page
                    putchar (' ');
                    j++;
                }
                else
                    // sinon affichage du caractre
                    putchar (bufSubject [j++]);
            }
        }
        // sinon (affichage avec le jeu de caractres UTF-8)
        else
        {
            // on affichera jusqu' sz_avantdate caractres maximum
            while (++i < sz_avantdate && bufSubject [j])
            {
                // si prsence d'une tabulation
                if (bufSubject [j] == '\t')
                {
                    // afficher un blanc pour une meilleure mise en page
                    putchar (' ');
                    j++;
                }
                // sinon affichage du caractre
                else
                {
                    // mais certains caractres ne seront pas compts
                    if ((bufSubject [j] & 0xC0) == 0x80)
                        i--;

                    putchar (bufSubject [j++]);
                }
            }
        }
    }
    else
        i++;    // pour corriger l'alignement

    // complt par des blancs
    while (i++ <= sz_avantdate)
        putchar (' ');

    // affichage de la date et de l'heure du message
    if (*bufDate)
    {
        j = 6;

        // jour du mois
        while (bufDate [j] < '0' || bufDate [j] > '9')
            j++;

        printf ("%02d", atoi (bufDate + j));

        while (bufDate [j] > ' ')
            j++;

        j++;

        // mois
        i = 0;

        while (i < 12 && (toupper (bufDate [j]) != mois [i][0]
                     || tolower (bufDate [j+1]) != mois [i][1]
                     || tolower (bufDate [j+2]) != mois [i][2]))
            i++;

        printf ("/%02d/", i + 1);

        // anne sur 2 chiffres
        while (bufDate [j] < '0' || bufDate [j] > '9')
            j++;

        printf ("%02d", atoi (bufDate + j) % 100);

        while (bufDate [j] > ' ')
            j++;

        // heure, sans les secondes ni le fuseau horaire
        for (i = 0; i < 6; i++)
            putchar (bufDate [j++]);
    }

    // passage  la ligne
    putchar ('\n');
}
